# -*- perl -*-
# DO NOT EDIT - This file is generated by UMMF; http://ummf.sourceforge.net 
# From template: $Id: Perl.txt,v 1.77 2006/05/14 01:40:03 kstephens Exp $

package UMMF::UML_1_5::Foundation::Core::Association;

#use 5.6.1;
use strict;
use warnings;

#################################################################
# Version
#

our $VERSION = do { my @r = (q{1.5} =~ /\d+/g); sprintf "%d." . "%03d" x $#r, @r };


#################################################################
# Documentation
#

=head1 NAME

UMMF::UML_1_5::Foundation::Core::Association -- 

=head1 VERSION

1.5

=head1 SYNOPSIS

=head1 DESCRIPTION 

=head1 USAGE

=head1 EXPORT

=head1 METATYPE

L<UMMF::UML_1_5::Foundation::Core::Class|UMMF::UML_1_5::Foundation::Core::Class>

=head1 SUPERCLASSES

L<UMMF::UML_1_5::Foundation::Core::GeneralizableElement|UMMF::UML_1_5::Foundation::Core::GeneralizableElement>

L<UMMF::UML_1_5::Foundation::Core::Relationship|UMMF::UML_1_5::Foundation::Core::Relationship>




=head1 ATTRIBUTES

I<NO ATTRIBUTES>


=head1 ASSOCIATIONS


=head2 C<association> : I<THIS> C<1> E<lt>----  C<> : UMMF::UML_1_5::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ClearAssociationAction C<0..1>



=over 4

=item metatype = L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML_1_5::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ClearAssociationAction|UMMF::UML_1_5::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ClearAssociationAction>

=item multiplicity = C<0..1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<0>

=item aggregation = C<none>

=item visibility = C<private>

=item container_type = C<Set::Object>

=back


=head2 C<base> : I<THIS> C<0..1> E<lt>---E<gt>  C<associationRole> : UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole C<0..*>



=over 4

=item metatype = L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole|UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole>

=item multiplicity = C<0..*>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<1>

=item aggregation = C<none>

=item visibility = C<private>

=item container_type = C<Set::Object>

=back


=head2 C<_association> : I<THIS> C<1> E<lt>---E<gt>  C<connection> : UMMF::UML_1_5::Foundation::Core::AssociationEnd C<2..*>



=over 4

=item metatype = L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>

=item multiplicity = C<2..*>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<ordered>

=item isNavigable = C<1>

=item aggregation = C<none>

=item visibility = C<public>

=item container_type = C<ARRAY>

=back


=head2 C<association> : I<THIS> C<1> E<lt>---E<gt>  C<link> : UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link C<0..*>



=over 4

=item metatype = L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link|UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link>

=item multiplicity = C<0..*>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<1>

=item aggregation = C<none>

=item visibility = C<private>

=item container_type = C<Set::Object>

=back



=head1 METHODS

=cut



#################################################################
# Dependencies
#





use Carp qw(croak confess);
use Set::Object 1.05;
use Class::Multimethods 1.70;
use Data::Dumper;
use Scalar::Util qw(weaken);
use UMMF::UML_1_5::__ObjectBase qw(:__ummf_array);


#################################################################
# Generalizations
#

use base qw(
  UMMF::UML_1_5::Foundation::Core::GeneralizableElement  UMMF::UML_1_5::Foundation::Core::Relationship



);


#################################################################
# Exports
#

our @EXPORT_OK = qw(
);
our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );





#################################################################
# Validation
#


=head2 C<__validate_type>

  UMMF::UML_1_5::Foundation::Core::Association->__validate_type($value);

Returns true if C<$value> is a valid representation of L<UMMF::UML_1_5::Foundation::Core::Association|UMMF::UML_1_5::Foundation::Core::Association>.

=cut
sub __validate_type($$)
{
  my ($self, $x) = @_;

  no warnings;

  UNIVERSAL::isa($x, 'UMMF::UML_1_5::Foundation::Core::Association')  ;
}


=head2 C<__typecheck>

  UMMF::UML_1_5::Foundation::Core::Association->__typecheck($value, $msg);

Calls C<confess()> with C<$msg> if C<<UMMF::UML_1_5::Foundation::Core::Association->__validate_type($value)>> is false.

=cut
sub __typecheck
{
  my ($self, $x, $msg) = @_;

  confess("typecheck: $msg: type '" . 'UMMF::UML_1_5::Foundation::Core::Association' . ": value '$x'")
    unless __validate_type($self, $x);
}


=head2 C<isaAssociation>


Returns true if receiver is a L<UMMF::UML_1_5::Foundation::Core::Association|UMMF::UML_1_5::Foundation::Core::Association>.
Other receivers will return false.

=cut
sub isaAssociation { 1 }


=head2 C<isaFoundation__Core__Association>


Returns true if receiver is a L<UMMF::UML_1_5::Foundation::Core::Association|UMMF::UML_1_5::Foundation::Core::Association>.
Other receivers will return false.
This is the fully qualified version of the C<isaAssociation> method.

=cut
sub isaFoundation__Core__Association { 1 }


#################################################################
# Introspection
#

=head2 C<__model_name> 

  my $name = $obj_or_package->__model_name;

Returns the UML Model name (C<'Foundation::Core::Association'>) for an object or package of
this Classifier.

=cut
sub __model_name { 'Foundation::Core::Association' }



=head2 C<__isAbstract>

  $package->__isAbstract;

Returns C<0>.

=cut
sub __isAbstract { 0; }


my $__tangram_schema;
=head2 C<__tangram_schema>

  my $tangram_schema $obj_or_package->__tangram_schema

Returns a HASH ref that describes this Classifier for Tangram.

See L<UMMF::Export::Perl::Tangram|UMMF::Export::Perl::Tangram>

=cut
sub __tangram_schema
{
  my ($self) = @_;

  $__tangram_schema ||=
  {
   'classes' =>
   [
     'UMMF::UML_1_5::Foundation::Core::Association' =>
     {
       'table' => 'Foundation__Core__Association',
       'abstract' => 0,
       'slots' => 
       { 
	 # Attributes
	 
	 # Associations
	 	 	                     	 	       'associationRole'
       => {
	 'type_impl' => 'iset',
         'class' => 'UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole',

                           'table' => 'Foundation__Core__Association__associationRole', 

                                                               'coll' => 'base',

                                                                               }
      ,
                  	 	       'connection'
       => {
	 'type_impl' => 'iarray',
         'class' => 'UMMF::UML_1_5::Foundation::Core::AssociationEnd',

                           'table' => 'Foundation__Core__Association__connection', 

                                                               'coll' => '_association',

                  'slot' => '_association_i', 

                                    'aggreg' => '1', 

                                           }
      ,
                  	 	       'link'
       => {
	 'type_impl' => 'iset',
         'class' => 'UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link',

                           'table' => 'Foundation__Core__Association__link', 

                                                               'coll' => 'association',

                                                                               }
      ,
                         },
       'bases' => [  'UMMF::UML_1_5::Foundation::Core::GeneralizableElement',  'UMMF::UML_1_5::Foundation::Core::Relationship',  ],
       'sql' => {

       },
     },
   ],

   'sql' =>
   {
    # Note Tangram::Ref::get_exporter() has
    # "UPDATE $table SET $self->{col} = $refid WHERE id = $id",
    # The id_col is hard-coded, 
    # Thus id_col will not work.
    #'id_col' => '__sid',
    #'class_col' => '__stype',
   },
     # 'set_id' => sub { }
     # 'get_id' => sub { }

      
  };
}


#################################################################
# Class Attributes
#


    

#################################################################
# Class Associations
#


    

#################################################################
# Initialization
#


=head2 C<___initialize>

Initialize all Attributes and AssociationEnds in a instance of this Classifier.
Does B<not> initalize slots in its Generalizations.

See also: C<__initialize>.

=cut
sub ___initialize
{
  my ($self) = @_;

  # Attributes



  # Associations

  # AssociationEnd 
  #  base 0..1
  #  <--> 
  #  associationRole 0..* UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole.
    if ( defined $self->{'associationRole'} ) {
    my $x = $self->{'associationRole'};
        $self->{'associationRole'} = Set::Object->new();
        $self->set_associationRole(@$x);
  }
  
  # AssociationEnd 
  #  _association 1
  #  <--> 
  #  connection 2..* UMMF::UML_1_5::Foundation::Core::AssociationEnd.
    if ( defined $self->{'connection'} ) {
    my $x = $self->{'connection'};
        $self->{'connection'} = [ ];
        $self->set_connection(@$x);
  }
  
  # AssociationEnd 
  #  association 1
  #  <--> 
  #  link 0..* UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link.
    if ( defined $self->{'link'} ) {
    my $x = $self->{'link'};
        $self->{'link'} = Set::Object->new();
        $self->set_link(@$x);
  }
  

  $self;
}


my $__initialize_use;

=head2 C<__initialize>

Initialize all slots in this Classifier and all its Generalizations.

See also: C<___initialize>.

=cut
sub __initialize
{
  my ($self) = @_;

  # $DB::single = 1;

  unless ( ! $__initialize_use ) {
    $__initialize_use = 1;
    $self->__use('UMMF::UML_1_5::Foundation::Core::GeneralizableElement');
    $self->__use('UMMF::UML_1_5::Foundation::Core::Element');
    $self->__use('UMMF::UML_1_5::Foundation::Core::ModelElement');
    $self->__use('UMMF::UML_1_5::Foundation::Core::Relationship');
  }

  $self->UMMF::UML_1_5::Foundation::Core::Association::___initialize;
  $self->UMMF::UML_1_5::Foundation::Core::GeneralizableElement::___initialize;
  $self->UMMF::UML_1_5::Foundation::Core::Element::___initialize;
  $self->UMMF::UML_1_5::Foundation::Core::ModelElement::___initialize;
  $self->UMMF::UML_1_5::Foundation::Core::Relationship::___initialize;

  $self;
}
      

=head2 C<__create>

Calls all <<create>> Methods for this Classifier and all Generalizations.

See also: C<___create>.

=cut
sub __create
{
  my ($self, @args) = @_;

  # $DB::single = 1;
  $self->UMMF::UML_1_5::Foundation::Core::Association::___create(@args);
  $self->UMMF::UML_1_5::Foundation::Core::GeneralizableElement::___create();
  $self->UMMF::UML_1_5::Foundation::Core::Element::___create();
  $self->UMMF::UML_1_5::Foundation::Core::ModelElement::___create();
  $self->UMMF::UML_1_5::Foundation::Core::Relationship::___create();

  $self;
}




#################################################################
# Attributes
#




#################################################################
# Association
#


=for html <hr/>

=cut

#################################################################
# AssociationEnd base <---> associationRole
# type = UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole
# multiplicity = 0..*
# ordering = 

=head2 C<associationRole>

  my @val = $obj->associationRole;
  my $ary_val = $obj->associationRole;

Returns the AssociationEnd C<associationRole> values of type L<UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole|UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole>.
In array context, returns all the objects in the Association.
In scalar context, returns an array ref of all the objects in the Association.

=cut
sub associationRole ($)
{
  my ($self) = @_;

    my $x = $self->{'associationRole'};

  # confess("Container for associationRole $x is not a blessed ref: " . Data::Dumper->new([ $self ], [qw($self)])->Maxdepth(2)->Dump()) if $x && ref($x) !~ /::/;
 
  wantarray ? ($x ? $x->members() : ()) : [ $x ? $x->members() : () ];
  
}


=head2 C<set_associationRole>

  $obj->set_associationRole(@val);

Sets the AssociationEnd C<associationRole> value.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole|UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole>.
Returns C<$obj>.

=cut
sub set_associationRole ($@)
{
  my ($self, @val) = @_;
  
  $self->clear_associationRole;
  $self->add_associationRole(@val);
}


=head2 C<add_associationRole>

  $obj->add_associationRole(@val);

Adds AssociationEnd C<associationRole> values.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole|UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole>.
Returns C<$obj>.

=cut
sub add_associationRole ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'associationRole'} ||= Set::Object->new();
    my $old; # Place holder for other MACRO.
  
  for my $val ( @val ) {
    # Recursion lock
        next if $x->includes($val);
        $self->__use('UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole')->__typecheck($val, "UMMF::UML_1_5::Foundation::Core::Association.associationRole");

    # Recursion lock
        $x->insert($val);
    # weaken?
    
    # Remove and add associations with other ends.
        
    $old->remove_base($self) if $old;
    $val->add_base($self)    if $val;

    }
  
  $self;
}


=head2 C<remove_associationRole>

  $obj->remove_associationRole(@val);

Removes the AssociationEnd C<associationRole> values C<@val>.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole|UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole>.
Returns C<$obj>.

=cut
sub remove_associationRole ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'associationRole'} ||= Set::Object->new();
  
  for my $old ( @val ) {
    # Recursion lock
        next unless $x->includes($old);
    
    my $val = $old;
      
    $self->__use('UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole')->__typecheck($val, "UMMF::UML_1_5::Foundation::Core::Association.associationRole");

    # Recursion lock
        $x->remove($old);
    
    $val = undef;

    # Remove associations with other ends.

        
    $old->remove_base($self) if $old;
    $val->add_base($self)    if $val;

  ;

  }
  
  $self;
}


=head2 C<clear_associationRole>

  $obj->clear_associationRole;

Clears the AssociationEnd C<associationRole> links to L<UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole|UMMF::UML_1_5::Behavioral_Elements::Collaborations::AssociationRole>.
Returns C<$obj>.

=cut
sub clear_associationRole ($) 
{
  my ($self) = @_;
  
    my $x = $self->{'associationRole'} ||= Set::Object->new();
  
  my $val; # Place holder for other MACRO.
  
    $self->{'associationRole'} = Set::Object->new(); # Recursion lock
  for my $old ( $x->members() ) {     # Recursion lock
  
    # Remove associations with other ends.

        
    $old->remove_base($self) if $old;
    $val->add_base($self)    if $val;

  ;

  }
  
  $self;
}


=head2 C<count_associationRole>

  $obj->count_associationRole;

Returns the number of elements associated with C<associationRole>.

=cut
sub count_associationRole ($)
{
  my ($self) = @_;

  my $x = $self->{'associationRole'};

    defined $x ? $x->size : 0;
  }




=for html <hr/>

=cut

#################################################################
# AssociationEnd _association <---> connection
# type = UMMF::UML_1_5::Foundation::Core::AssociationEnd
# multiplicity = 2..*
# ordering = ordered

=head2 C<connection>

  my @val = $obj->connection;
  my $ary_val = $obj->connection;

Returns the AssociationEnd C<connection> values of type L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>.
In array context, returns all the objects in the Association.
In scalar context, returns an array ref of all the objects in the Association.

=cut
sub connection ($)
{
  my ($self) = @_;

    my $x = $self->{'connection'} ||= [ ];

  wantarray ? @{$x} : $x;
  
}


=head2 C<index_connection>

  my $x = $obj->index_connection($i);
  my @x = $obj->index_connection($i, $count);

In scalar context, returns the value of AssociationEnd C<connection> at index C<$i>.
In array context, returns the values between index C<$i> and C<$i + $count - 1>, inclusive.

=cut
sub index_connection ($$@)
{
  my ($self, $i, $count) = @_;

  ;

  my $val = $self->{'connection'} ||= [ ];

  ;

  wantarray ? $val->[$i .. (defined $count ? $i + $count - 1 : $i)]
            : $val->[$i];
}


=head2 C<index_of_connection>

  my $index = $obj->index_of_connection($val);

Returns the index of C<$val> in AssociationEnd C<connection>.
Return C<undef> if C<$val> is not in C<connection>.

=cut
sub index_of_connection ($$)
{
  my ($self, $x) = @_;

  ;

  my $val = $self->{'connection'} ||= [ ];

  ;

  __ummf_array_index($val, $x);
}


=head2 C<set_connection>

  $obj->set_connection(@val);

Sets the AssociationEnd C<connection> value.
Elements of C<@val> must of type L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>.
Returns C<$obj>.

=cut
sub set_connection ($@)
{
  my ($self, @val) = @_;
  
  $self->clear_connection;
  $self->add_connection(@val);
}


=head2 C<set_index_connection>

  $obj->set_index_connection($i, $val);

Sets the value of AssociationEnd C<connection> at index C<$i>.
Returns self.

=cut
sub set_index_connection ($$$)
{
  my ($self, $i, $val) = @_;

  ;

  my $x = $self->{'connection'} ||= [ ];

  no warnings;
  my $old;
  if ( ($old = $x->[$i]) ne $val) {
    # Recursion lock
        $x->[$i] = $val
    ;

    # Remove and add associations with other ends.
        
    $old->remove__association($self) if $old;
    $val->add__association($self)    if $val;

  
    ;
  }

  $self;
}


=head2 C<add_connection>

  $obj->add_connection(@val);

Adds AssociationEnd C<connection> values.
Elements of C<@val> must of type L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>.
Returns C<$obj>.

=cut
sub add_connection ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'connection'} ||= [ ];
    my $old; # Place holder for other MACRO.
  
  for my $val ( @val ) {
    # Recursion lock
        next if grep($_ eq $val, @$x);
        $self->__use('UMMF::UML_1_5::Foundation::Core::AssociationEnd')->__typecheck($val, "UMMF::UML_1_5::Foundation::Core::Association.connection");

    # Recursion lock
        push(@{$x}, $val);
        
    # Remove and add associations with other ends.
        
    $old->remove__association($self) if $old;
    $val->add__association($self)    if $val;

    }
  
  $self;
}


=head2 C<add_index_connection>

  $obj->add_index_connection($i, @val);

Adds AssociationEnd C<connection> values at index C<$i>.
Elements of C<@val> must of type L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>.
Returns C<$obj>.

=cut
sub add_index_connection ($$@)
{
  my ($self, $i, @val) = @_;

  
    my $x = $self->{'connection'} ||= [ ];
    my $old; # Place holder for other MACRO.
  
  for my $val ( @val ) {
    # Recursion lock
        next if grep($_ eq $val, @$x);
        $self->__use('UMMF::UML_1_5::Foundation::Core::AssociationEnd')->__typecheck($val, "UMMF::UML_1_5::Foundation::Core::Association.connection");

    # Recursion lock
        splice(@{$x}, $i, 0, $val); # Recursion lock
        ++ $i;
    
    # Remove and add associations with other ends.
        
    $old->remove__association($self) if $old;
    $val->add__association($self)    if $val;

    }
  
  
  $self;
}


=head2 C<remove_connection>

  $obj->remove_connection(@val);

Removes the AssociationEnd C<connection> values C<@val>.
Elements of C<@val> must of type L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>.
Returns C<$obj>.

=cut
sub remove_connection ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'connection'} ||= [ ];
  
  for my $old ( @val ) {
    # Recursion lock
        my $i; # index of $old in @$x.
    next unless defined($i = __ummf_array_index($x, $old));
    
    my $val = $old;
      
    $self->__use('UMMF::UML_1_5::Foundation::Core::AssociationEnd')->__typecheck($val, "UMMF::UML_1_5::Foundation::Core::Association.connection");

    # Recursion lock
        splice(@$x, $i, 1);     
    $val = undef;

    # Remove associations with other ends.

        
    $old->remove__association($self) if $old;
    $val->add__association($self)    if $val;

  ;

  }
  
  $self;
}


=head2 C<clear_connection>

  $obj->clear_connection;

Clears the AssociationEnd C<connection> links to L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>.
Returns C<$obj>.

=cut
sub clear_connection ($) 
{
  my ($self) = @_;
  
    my $x = $self->{'connection'} ||= [ ];
  
  my $val; # Place holder for other MACRO.
  
    $self->{'connection'} = [ ];  # Recursion lock
  for my $old ( @$x ) { # Recursion lock
  
    # Remove associations with other ends.

        
    $old->remove__association($self) if $old;
    $val->add__association($self)    if $val;

  ;

  }
  
  $self;
}


=head2 C<count_connection>

  $obj->count_connection;

Returns the number of elements associated with C<connection>.

=cut
sub count_connection ($)
{
  my ($self) = @_;

  my $x = $self->{'connection'};

    defined $x ? scalar @$x : 0;
  }




=for html <hr/>

=cut

#################################################################
# AssociationEnd association <---> link
# type = UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link
# multiplicity = 0..*
# ordering = 

=head2 C<link>

  my @val = $obj->link;
  my $ary_val = $obj->link;

Returns the AssociationEnd C<link> values of type L<UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link|UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link>.
In array context, returns all the objects in the Association.
In scalar context, returns an array ref of all the objects in the Association.

=cut
sub link ($)
{
  my ($self) = @_;

    my $x = $self->{'link'};

  # confess("Container for link $x is not a blessed ref: " . Data::Dumper->new([ $self ], [qw($self)])->Maxdepth(2)->Dump()) if $x && ref($x) !~ /::/;
 
  wantarray ? ($x ? $x->members() : ()) : [ $x ? $x->members() : () ];
  
}


=head2 C<set_link>

  $obj->set_link(@val);

Sets the AssociationEnd C<link> value.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link|UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link>.
Returns C<$obj>.

=cut
sub set_link ($@)
{
  my ($self, @val) = @_;
  
  $self->clear_link;
  $self->add_link(@val);
}


=head2 C<add_link>

  $obj->add_link(@val);

Adds AssociationEnd C<link> values.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link|UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link>.
Returns C<$obj>.

=cut
sub add_link ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'link'} ||= Set::Object->new();
    my $old; # Place holder for other MACRO.
  
  for my $val ( @val ) {
    # Recursion lock
        next if $x->includes($val);
        $self->__use('UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link')->__typecheck($val, "UMMF::UML_1_5::Foundation::Core::Association.link");

    # Recursion lock
        $x->insert($val);
    # weaken?
    
    # Remove and add associations with other ends.
        
    $old->remove_association($self) if $old;
    $val->add_association($self)    if $val;

    }
  
  $self;
}


=head2 C<remove_link>

  $obj->remove_link(@val);

Removes the AssociationEnd C<link> values C<@val>.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link|UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link>.
Returns C<$obj>.

=cut
sub remove_link ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'link'} ||= Set::Object->new();
  
  for my $old ( @val ) {
    # Recursion lock
        next unless $x->includes($old);
    
    my $val = $old;
      
    $self->__use('UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link')->__typecheck($val, "UMMF::UML_1_5::Foundation::Core::Association.link");

    # Recursion lock
        $x->remove($old);
    
    $val = undef;

    # Remove associations with other ends.

        
    $old->remove_association($self) if $old;
    $val->add_association($self)    if $val;

  ;

  }
  
  $self;
}


=head2 C<clear_link>

  $obj->clear_link;

Clears the AssociationEnd C<link> links to L<UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link|UMMF::UML_1_5::Behavioral_Elements::Common_Behavior::Link>.
Returns C<$obj>.

=cut
sub clear_link ($) 
{
  my ($self) = @_;
  
    my $x = $self->{'link'} ||= Set::Object->new();
  
  my $val; # Place holder for other MACRO.
  
    $self->{'link'} = Set::Object->new(); # Recursion lock
  for my $old ( $x->members() ) {     # Recursion lock
  
    # Remove associations with other ends.

        
    $old->remove_association($self) if $old;
    $val->add_association($self)    if $val;

  ;

  }
  
  $self;
}


=head2 C<count_link>

  $obj->count_link;

Returns the number of elements associated with C<link>.

=cut
sub count_link ($)
{
  my ($self) = @_;

  my $x = $self->{'link'};

    defined $x ? $x->size : 0;
  }







# End of Class Association


=pod

=for html <hr/>

I<END OF DOCUMENT>

=cut

############################################################################

1; # is true!

############################################################################

### Keep these comments at end of file: kstephens@users.sourceforge.net 2003/04/06 ###
### Local Variables: ###
### mode:perl ###
### perl-indent-level:2 ###
### perl-continued-statement-offset:0 ###
### perl-brace-offset:0 ###
### perl-label-offset:0 ###
### End: ###

