# -*- perl -*-
# DO NOT EDIT - This file is generated by UMMF; http://ummf.sourceforge.net 
# From template: $Id: Perl.txt,v 1.77 2006/05/14 01:40:03 kstephens Exp $

package UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause;

#use 5.6.1;
use strict;
use warnings;

#################################################################
# Version
#

our $VERSION = do { my @r = (q{1.5} =~ /\d+/g); sprintf "%d." . "%03d" x $#r, @r };


#################################################################
# Documentation
#

=head1 NAME

UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause -- 

=head1 VERSION

1.5

=head1 SYNOPSIS

=head1 DESCRIPTION 

=head1 USAGE

=head1 EXPORT

=head1 METATYPE

L<UMMF::UML_1_5::Foundation::Core::Class|UMMF::UML_1_5::Foundation::Core::Class>

=head1 SUPERCLASSES

L<UMMF::UML_1_5::Foundation::Core::Element|UMMF::UML_1_5::Foundation::Core::Element>




=head1 ATTRIBUTES

I<NO ATTRIBUTES>


=head1 ASSOCIATIONS


=head2 C<> : I<THIS> C<0..1> ----E<gt>  C<body> : UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action C<1>



=over 4

=item metatype = L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action>

=item multiplicity = C<1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<1>

=item aggregation = C<none>

=item visibility = C<public>

=item container_type = C<Set::Object>

=back


=head2 C<> : I<THIS> C<0..*> ----E<gt>  C<bodyOutput> : UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin C<0..*>



=over 4

=item metatype = L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin>

=item multiplicity = C<0..*>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<ordered>

=item isNavigable = C<1>

=item aggregation = C<none>

=item visibility = C<public>

=item container_type = C<ARRAY>

=back


=head2 C<successorClause> : I<THIS> C<0..*> E<lt>---E<gt>  C<predecessorClause> : UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause C<0..*>



=over 4

=item metatype = L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause>

=item multiplicity = C<0..*>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<1>

=item aggregation = C<none>

=item visibility = C<public>

=item container_type = C<Set::Object>

=back


=head2 C<predecessorClause> : I<THIS> C<0..*> E<lt>---E<gt>  C<successorClause> : UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause C<0..*>



=over 4

=item metatype = L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause>

=item multiplicity = C<0..*>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<1>

=item aggregation = C<none>

=item visibility = C<public>

=item container_type = C<Set::Object>

=back


=head2 C<> : I<THIS> C<0..1> ----E<gt>  C<test> : UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action C<1>



=over 4

=item metatype = L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action>

=item multiplicity = C<1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<1>

=item aggregation = C<none>

=item visibility = C<public>

=item container_type = C<Set::Object>

=back


=head2 C<> : I<THIS> C<0..*> ----E<gt>  C<testOutput> : UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin C<0..*>



=over 4

=item metatype = L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin>

=item multiplicity = C<0..*>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<1>

=item aggregation = C<none>

=item visibility = C<public>

=item container_type = C<Set::Object>

=back



=head1 METHODS

=cut



#################################################################
# Dependencies
#





use Carp qw(croak confess);
use Set::Object 1.05;
use Class::Multimethods 1.70;
use Data::Dumper;
use Scalar::Util qw(weaken);
use UMMF::UML_1_5::__ObjectBase qw(:__ummf_array);


#################################################################
# Generalizations
#

use base qw(
  UMMF::UML_1_5::Foundation::Core::Element



);


#################################################################
# Exports
#

our @EXPORT_OK = qw(
);
our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );





#################################################################
# Validation
#


=head2 C<__validate_type>

  UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause->__validate_type($value);

Returns true if C<$value> is a valid representation of L<UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause>.

=cut
sub __validate_type($$)
{
  my ($self, $x) = @_;

  no warnings;

  UNIVERSAL::isa($x, 'UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause')  ;
}


=head2 C<__typecheck>

  UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause->__typecheck($value, $msg);

Calls C<confess()> with C<$msg> if C<<UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause->__validate_type($value)>> is false.

=cut
sub __typecheck
{
  my ($self, $x, $msg) = @_;

  confess("typecheck: $msg: type '" . 'UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause' . ": value '$x'")
    unless __validate_type($self, $x);
}


=head2 C<isaClause>


Returns true if receiver is a L<UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause>.
Other receivers will return false.

=cut
sub isaClause { 1 }


=head2 C<isaBehavioral_Elements__Actions__Composite_Actions__Clause>


Returns true if receiver is a L<UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause>.
Other receivers will return false.
This is the fully qualified version of the C<isaClause> method.

=cut
sub isaBehavioral_Elements__Actions__Composite_Actions__Clause { 1 }


#################################################################
# Introspection
#

=head2 C<__model_name> 

  my $name = $obj_or_package->__model_name;

Returns the UML Model name (C<'Behavioral_Elements::Actions::Composite_Actions::Clause'>) for an object or package of
this Classifier.

=cut
sub __model_name { 'Behavioral_Elements::Actions::Composite_Actions::Clause' }



=head2 C<__isAbstract>

  $package->__isAbstract;

Returns C<0>.

=cut
sub __isAbstract { 0; }


my $__tangram_schema;
=head2 C<__tangram_schema>

  my $tangram_schema $obj_or_package->__tangram_schema

Returns a HASH ref that describes this Classifier for Tangram.

See L<UMMF::Export::Perl::Tangram|UMMF::Export::Perl::Tangram>

=cut
sub __tangram_schema
{
  my ($self) = @_;

  $__tangram_schema ||=
  {
   'classes' =>
   [
     'UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause' =>
     {
       'table' => 'Behavioral_Elements__Actions__Composite_Actions__Clause',
       'abstract' => 0,
       'slots' => 
       { 
	 # Attributes
	 
	 # Associations
	 	 	       'body'
       => {
	 'type_impl' => 'ref',
         'class' => 'UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action',

                                             'col' => 'body', 

                                                                                 'aggreg' => '1', 

                                           }
      ,
                  	 	       'bodyOutput'
       => {
	 'type_impl' => 'array',
         'class' => 'UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin',

                           'table' => 'Behavioral_Elements__Actions__Clause_BodyOutput', 

                                                      'item' => 'bodyOutput', 

                  'coll' => 'clause',

                  'slot' => 'bodyOutput_i', 

                                                                      }
      ,
                  	 	       'predecessorClause'
       => {
	 'type_impl' => 'set',
         'class' => 'UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause',

                           'table' => 'Behavioral_Elements__Actions__Composite_Actions__Clause__SuccessorClause_PredecessorClause', 

                                                      'item' => 'predecessorClause', 

                  'coll' => 'successorClause',

                                                                               }
      ,
                  	 	       'successorClause'
       => {
	 'type_impl' => 'set',
         'class' => 'UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause',

                           'table' => 'Behavioral_Elements__Actions__Composite_Actions__Clause__SuccessorClause_PredecessorClause', 

                                                      'item' => 'successorClause', 

                  'coll' => 'predecessorClause',

                                                                               }
      ,
                  	 	       'test'
       => {
	 'type_impl' => 'ref',
         'class' => 'UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action',

                                             'col' => 'test', 

                                                                                 'aggreg' => '1', 

                                           }
      ,
                  	 	       'testOutput'
       => {
	 'type_impl' => 'set',
         'class' => 'UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin',

                           'table' => 'Behavioral_Elements__Actions__Clause_TestOutput', 

                                                      'item' => 'testOutput', 

                  'coll' => 'clause',

                                                                               }
      ,
                         },
       'bases' => [  'UMMF::UML_1_5::Foundation::Core::Element',  ],
       'sql' => {

       },
     },
   ],

   'sql' =>
   {
    # Note Tangram::Ref::get_exporter() has
    # "UPDATE $table SET $self->{col} = $refid WHERE id = $id",
    # The id_col is hard-coded, 
    # Thus id_col will not work.
    #'id_col' => '__sid',
    #'class_col' => '__stype',
   },
     # 'set_id' => sub { }
     # 'get_id' => sub { }

      
  };
}


#################################################################
# Class Attributes
#


    

#################################################################
# Class Associations
#


    

#################################################################
# Initialization
#


=head2 C<___initialize>

Initialize all Attributes and AssociationEnds in a instance of this Classifier.
Does B<not> initalize slots in its Generalizations.

See also: C<__initialize>.

=cut
sub ___initialize
{
  my ($self) = @_;

  # Attributes



  # Associations

  # AssociationEnd 
  #   0..1
  #  <--> 
  #  body 1 UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action.
    if ( defined $self->{'body'} ) {
    my $x = $self->{'body'};
    $self->{'body'} = undef;
    $self->set_body($x);
  }
  
  # AssociationEnd 
  #   0..*
  #  <--> 
  #  bodyOutput 0..* UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin.
    if ( defined $self->{'bodyOutput'} ) {
    my $x = $self->{'bodyOutput'};
        $self->{'bodyOutput'} = [ ];
        $self->set_bodyOutput(@$x);
  }
  
  # AssociationEnd 
  #  successorClause 0..*
  #  <--> 
  #  predecessorClause 0..* UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause.
    if ( defined $self->{'predecessorClause'} ) {
    my $x = $self->{'predecessorClause'};
        $self->{'predecessorClause'} = Set::Object->new();
        $self->set_predecessorClause(@$x);
  }
  
  # AssociationEnd 
  #  predecessorClause 0..*
  #  <--> 
  #  successorClause 0..* UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause.
    if ( defined $self->{'successorClause'} ) {
    my $x = $self->{'successorClause'};
        $self->{'successorClause'} = Set::Object->new();
        $self->set_successorClause(@$x);
  }
  
  # AssociationEnd 
  #   0..1
  #  <--> 
  #  test 1 UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action.
    if ( defined $self->{'test'} ) {
    my $x = $self->{'test'};
    $self->{'test'} = undef;
    $self->set_test($x);
  }
  
  # AssociationEnd 
  #   0..*
  #  <--> 
  #  testOutput 0..* UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin.
    if ( defined $self->{'testOutput'} ) {
    my $x = $self->{'testOutput'};
        $self->{'testOutput'} = Set::Object->new();
        $self->set_testOutput(@$x);
  }
  

  $self;
}


my $__initialize_use;

=head2 C<__initialize>

Initialize all slots in this Classifier and all its Generalizations.

See also: C<___initialize>.

=cut
sub __initialize
{
  my ($self) = @_;

  # $DB::single = 1;

  unless ( ! $__initialize_use ) {
    $__initialize_use = 1;
    $self->__use('UMMF::UML_1_5::Foundation::Core::Element');
  }

  $self->UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause::___initialize;
  $self->UMMF::UML_1_5::Foundation::Core::Element::___initialize;

  $self;
}
      

=head2 C<__create>

Calls all <<create>> Methods for this Classifier and all Generalizations.

See also: C<___create>.

=cut
sub __create
{
  my ($self, @args) = @_;

  # $DB::single = 1;
  $self->UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause::___create(@args);
  $self->UMMF::UML_1_5::Foundation::Core::Element::___create();

  $self;
}




#################################################################
# Attributes
#




#################################################################
# Association
#


=for html <hr/>

=cut

#################################################################
# AssociationEnd  <---> body
# type = UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action
# multiplicity = 1
# ordering = 

=head2 C<body>

  my $val = $obj->body;

Returns the AssociationEnd C<body> value of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action>.

=cut
sub body ($)
{
  my ($self) = @_;
		  
  $self->{'body'};
}


=head2 C<set_body>

  $obj->set_body($val);

Sets the AssociationEnd C<body> value.
C<$val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action>.
Returns C<$obj>.

=cut
sub set_body ($$)
{
  my ($self, $val) = @_;
		  
  no warnings; # Use of uninitialized value in string ne at ...
		  
  my $old;
  if ( ($old = $self->{'body'}) ne $val ) { # Recursion lock

    if ( defined $val ) { $self->__use('UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action')->__typecheck($val, "UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause.body") }

    # Recursion lock
        $self->{'body'} = $val
    ;

    # Remove and add associations with other ends.
          }
		  
  $self;
}


=head2 C<add_body>

  $obj->add_body($val);

Adds the AssociationEnd C<body> value.
C<$val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action>.
Throws exception if a value already exists.
Returns C<$obj>.

=cut
sub add_body ($$)
{
  my ($self, $val) = @_;

  no warnings; # Use of uninitialized value in string ne at ...

  my $old;
  if ( ($old = $self->{'body'}) ne $val ) { # Recursion lock
    $self->__use('UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action')->__typecheck($val, "UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause.body");
      
    # confess("UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause::body: too many")
    # if defined $self->{'body'};

    # Recursion lock
        $self->{'body'} = $val
    ;

    # Remove and add associations with other ends.
        
  }

  $self;
}


=head2 C<remove_body>

  $obj->remove_body($val);

Removes the AssociationEnd C<body> value C<$val>.
Returns C<$obj>.

=cut
sub remove_body ($$)
{
  my ($self, $val) = @_;

  no warnings; # Use of uninitialized value in string ne at ...

  my $old;
  if ( ($old = $self->{'body'}) eq $val ) { # Recursion lock
    $val = $self->{'body'} = undef;         # Recursion lock

    # Remove and add associations with other ends.
        
  }
}


=head2 C<clear_body>

  $obj->clear_body;

Clears the AssociationEnd C<body> links to L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action>.
Returns C<$obj>.

=cut
sub clear_body ($@)
{
  my ($self) = @_;

  my $old;
  if ( defined ($old = $self->{'body'}) ) { # Recursion lock
    my $val = $self->{'body'} = undef;      # Recursion lock

    # Remove and add associations with other ends.
          }

  $self;
}


=head2 C<count_body>

  $obj->count_body;

Returns the number of elements of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action> associated with C<body>.

=cut
sub count_body ($)
{
  my ($self) = @_;

  my $x = $self->{'body'};

  defined $x ? 1 : 0;
}




=for html <hr/>

=cut

#################################################################
# AssociationEnd  <---> bodyOutput
# type = UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin
# multiplicity = 0..*
# ordering = ordered

=head2 C<bodyOutput>

  my @val = $obj->bodyOutput;
  my $ary_val = $obj->bodyOutput;

Returns the AssociationEnd C<bodyOutput> values of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin>.
In array context, returns all the objects in the Association.
In scalar context, returns an array ref of all the objects in the Association.

=cut
sub bodyOutput ($)
{
  my ($self) = @_;

    my $x = $self->{'bodyOutput'} ||= [ ];

  wantarray ? @{$x} : $x;
  
}


=head2 C<index_bodyOutput>

  my $x = $obj->index_bodyOutput($i);
  my @x = $obj->index_bodyOutput($i, $count);

In scalar context, returns the value of AssociationEnd C<bodyOutput> at index C<$i>.
In array context, returns the values between index C<$i> and C<$i + $count - 1>, inclusive.

=cut
sub index_bodyOutput ($$@)
{
  my ($self, $i, $count) = @_;

  ;

  my $val = $self->{'bodyOutput'} ||= [ ];

  ;

  wantarray ? $val->[$i .. (defined $count ? $i + $count - 1 : $i)]
            : $val->[$i];
}


=head2 C<index_of_bodyOutput>

  my $index = $obj->index_of_bodyOutput($val);

Returns the index of C<$val> in AssociationEnd C<bodyOutput>.
Return C<undef> if C<$val> is not in C<bodyOutput>.

=cut
sub index_of_bodyOutput ($$)
{
  my ($self, $x) = @_;

  ;

  my $val = $self->{'bodyOutput'} ||= [ ];

  ;

  __ummf_array_index($val, $x);
}


=head2 C<set_bodyOutput>

  $obj->set_bodyOutput(@val);

Sets the AssociationEnd C<bodyOutput> value.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin>.
Returns C<$obj>.

=cut
sub set_bodyOutput ($@)
{
  my ($self, @val) = @_;
  
  $self->clear_bodyOutput;
  $self->add_bodyOutput(@val);
}


=head2 C<set_index_bodyOutput>

  $obj->set_index_bodyOutput($i, $val);

Sets the value of AssociationEnd C<bodyOutput> at index C<$i>.
Returns self.

=cut
sub set_index_bodyOutput ($$$)
{
  my ($self, $i, $val) = @_;

  ;

  my $x = $self->{'bodyOutput'} ||= [ ];

  no warnings;
  my $old;
  if ( ($old = $x->[$i]) ne $val) {
    # Recursion lock
        $x->[$i] = $val
    ;

    # Remove and add associations with other ends.
        
    ;
  }

  $self;
}


=head2 C<add_bodyOutput>

  $obj->add_bodyOutput(@val);

Adds AssociationEnd C<bodyOutput> values.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin>.
Returns C<$obj>.

=cut
sub add_bodyOutput ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'bodyOutput'} ||= [ ];
    my $old; # Place holder for other MACRO.
  
  for my $val ( @val ) {
    # Recursion lock
        next if grep($_ eq $val, @$x);
        $self->__use('UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin')->__typecheck($val, "UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause.bodyOutput");

    # Recursion lock
        push(@{$x}, $val);
        
    # Remove and add associations with other ends.
          }
  
  $self;
}


=head2 C<add_index_bodyOutput>

  $obj->add_index_bodyOutput($i, @val);

Adds AssociationEnd C<bodyOutput> values at index C<$i>.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin>.
Returns C<$obj>.

=cut
sub add_index_bodyOutput ($$@)
{
  my ($self, $i, @val) = @_;

  
    my $x = $self->{'bodyOutput'} ||= [ ];
    my $old; # Place holder for other MACRO.
  
  for my $val ( @val ) {
    # Recursion lock
        next if grep($_ eq $val, @$x);
        $self->__use('UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin')->__typecheck($val, "UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause.bodyOutput");

    # Recursion lock
        splice(@{$x}, $i, 0, $val); # Recursion lock
        ++ $i;
    
    # Remove and add associations with other ends.
          }
  
  
  $self;
}


=head2 C<remove_bodyOutput>

  $obj->remove_bodyOutput(@val);

Removes the AssociationEnd C<bodyOutput> values C<@val>.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin>.
Returns C<$obj>.

=cut
sub remove_bodyOutput ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'bodyOutput'} ||= [ ];
  
  for my $old ( @val ) {
    # Recursion lock
        my $i; # index of $old in @$x.
    next unless defined($i = __ummf_array_index($x, $old));
    
    my $val = $old;
      
    $self->__use('UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin')->__typecheck($val, "UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause.bodyOutput");

    # Recursion lock
        splice(@$x, $i, 1);     
    $val = undef;

    # Remove associations with other ends.

        ;

  }
  
  $self;
}


=head2 C<clear_bodyOutput>

  $obj->clear_bodyOutput;

Clears the AssociationEnd C<bodyOutput> links to L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin>.
Returns C<$obj>.

=cut
sub clear_bodyOutput ($) 
{
  my ($self) = @_;
  
    my $x = $self->{'bodyOutput'} ||= [ ];
  
  my $val; # Place holder for other MACRO.
  
    $self->{'bodyOutput'} = [ ];  # Recursion lock
  for my $old ( @$x ) { # Recursion lock
  
    # Remove associations with other ends.

        ;

  }
  
  $self;
}


=head2 C<count_bodyOutput>

  $obj->count_bodyOutput;

Returns the number of elements associated with C<bodyOutput>.

=cut
sub count_bodyOutput ($)
{
  my ($self) = @_;

  my $x = $self->{'bodyOutput'};

    defined $x ? scalar @$x : 0;
  }




=for html <hr/>

=cut

#################################################################
# AssociationEnd successorClause <---> predecessorClause
# type = UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause
# multiplicity = 0..*
# ordering = 

=head2 C<predecessorClause>

  my @val = $obj->predecessorClause;
  my $ary_val = $obj->predecessorClause;

Returns the AssociationEnd C<predecessorClause> values of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause>.
In array context, returns all the objects in the Association.
In scalar context, returns an array ref of all the objects in the Association.

=cut
sub predecessorClause ($)
{
  my ($self) = @_;

    my $x = $self->{'predecessorClause'};

  # confess("Container for predecessorClause $x is not a blessed ref: " . Data::Dumper->new([ $self ], [qw($self)])->Maxdepth(2)->Dump()) if $x && ref($x) !~ /::/;
 
  wantarray ? ($x ? $x->members() : ()) : [ $x ? $x->members() : () ];
  
}


=head2 C<set_predecessorClause>

  $obj->set_predecessorClause(@val);

Sets the AssociationEnd C<predecessorClause> value.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause>.
Returns C<$obj>.

=cut
sub set_predecessorClause ($@)
{
  my ($self, @val) = @_;
  
  $self->clear_predecessorClause;
  $self->add_predecessorClause(@val);
}


=head2 C<add_predecessorClause>

  $obj->add_predecessorClause(@val);

Adds AssociationEnd C<predecessorClause> values.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause>.
Returns C<$obj>.

=cut
sub add_predecessorClause ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'predecessorClause'} ||= Set::Object->new();
    my $old; # Place holder for other MACRO.
  
  for my $val ( @val ) {
    # Recursion lock
        next if $x->includes($val);
        $self->__use('UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause')->__typecheck($val, "UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause.predecessorClause");

    # Recursion lock
        $x->insert($val);
    # weaken?
    
    # Remove and add associations with other ends.
        
    $old->remove_successorClause($self) if $old;
    $val->add_successorClause($self)    if $val;

    }
  
  $self;
}


=head2 C<remove_predecessorClause>

  $obj->remove_predecessorClause(@val);

Removes the AssociationEnd C<predecessorClause> values C<@val>.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause>.
Returns C<$obj>.

=cut
sub remove_predecessorClause ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'predecessorClause'} ||= Set::Object->new();
  
  for my $old ( @val ) {
    # Recursion lock
        next unless $x->includes($old);
    
    my $val = $old;
      
    $self->__use('UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause')->__typecheck($val, "UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause.predecessorClause");

    # Recursion lock
        $x->remove($old);
    
    $val = undef;

    # Remove associations with other ends.

        
    $old->remove_successorClause($self) if $old;
    $val->add_successorClause($self)    if $val;

  ;

  }
  
  $self;
}


=head2 C<clear_predecessorClause>

  $obj->clear_predecessorClause;

Clears the AssociationEnd C<predecessorClause> links to L<UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause>.
Returns C<$obj>.

=cut
sub clear_predecessorClause ($) 
{
  my ($self) = @_;
  
    my $x = $self->{'predecessorClause'} ||= Set::Object->new();
  
  my $val; # Place holder for other MACRO.
  
    $self->{'predecessorClause'} = Set::Object->new(); # Recursion lock
  for my $old ( $x->members() ) {     # Recursion lock
  
    # Remove associations with other ends.

        
    $old->remove_successorClause($self) if $old;
    $val->add_successorClause($self)    if $val;

  ;

  }
  
  $self;
}


=head2 C<count_predecessorClause>

  $obj->count_predecessorClause;

Returns the number of elements associated with C<predecessorClause>.

=cut
sub count_predecessorClause ($)
{
  my ($self) = @_;

  my $x = $self->{'predecessorClause'};

    defined $x ? $x->size : 0;
  }




=for html <hr/>

=cut

#################################################################
# AssociationEnd predecessorClause <---> successorClause
# type = UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause
# multiplicity = 0..*
# ordering = 

=head2 C<successorClause>

  my @val = $obj->successorClause;
  my $ary_val = $obj->successorClause;

Returns the AssociationEnd C<successorClause> values of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause>.
In array context, returns all the objects in the Association.
In scalar context, returns an array ref of all the objects in the Association.

=cut
sub successorClause ($)
{
  my ($self) = @_;

    my $x = $self->{'successorClause'};

  # confess("Container for successorClause $x is not a blessed ref: " . Data::Dumper->new([ $self ], [qw($self)])->Maxdepth(2)->Dump()) if $x && ref($x) !~ /::/;
 
  wantarray ? ($x ? $x->members() : ()) : [ $x ? $x->members() : () ];
  
}


=head2 C<set_successorClause>

  $obj->set_successorClause(@val);

Sets the AssociationEnd C<successorClause> value.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause>.
Returns C<$obj>.

=cut
sub set_successorClause ($@)
{
  my ($self, @val) = @_;
  
  $self->clear_successorClause;
  $self->add_successorClause(@val);
}


=head2 C<add_successorClause>

  $obj->add_successorClause(@val);

Adds AssociationEnd C<successorClause> values.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause>.
Returns C<$obj>.

=cut
sub add_successorClause ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'successorClause'} ||= Set::Object->new();
    my $old; # Place holder for other MACRO.
  
  for my $val ( @val ) {
    # Recursion lock
        next if $x->includes($val);
        $self->__use('UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause')->__typecheck($val, "UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause.successorClause");

    # Recursion lock
        $x->insert($val);
    # weaken?
    
    # Remove and add associations with other ends.
        
    $old->remove_predecessorClause($self) if $old;
    $val->add_predecessorClause($self)    if $val;

    }
  
  $self;
}


=head2 C<remove_successorClause>

  $obj->remove_successorClause(@val);

Removes the AssociationEnd C<successorClause> values C<@val>.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause>.
Returns C<$obj>.

=cut
sub remove_successorClause ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'successorClause'} ||= Set::Object->new();
  
  for my $old ( @val ) {
    # Recursion lock
        next unless $x->includes($old);
    
    my $val = $old;
      
    $self->__use('UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause')->__typecheck($val, "UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause.successorClause");

    # Recursion lock
        $x->remove($old);
    
    $val = undef;

    # Remove associations with other ends.

        
    $old->remove_predecessorClause($self) if $old;
    $val->add_predecessorClause($self)    if $val;

  ;

  }
  
  $self;
}


=head2 C<clear_successorClause>

  $obj->clear_successorClause;

Clears the AssociationEnd C<successorClause> links to L<UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause>.
Returns C<$obj>.

=cut
sub clear_successorClause ($) 
{
  my ($self) = @_;
  
    my $x = $self->{'successorClause'} ||= Set::Object->new();
  
  my $val; # Place holder for other MACRO.
  
    $self->{'successorClause'} = Set::Object->new(); # Recursion lock
  for my $old ( $x->members() ) {     # Recursion lock
  
    # Remove associations with other ends.

        
    $old->remove_predecessorClause($self) if $old;
    $val->add_predecessorClause($self)    if $val;

  ;

  }
  
  $self;
}


=head2 C<count_successorClause>

  $obj->count_successorClause;

Returns the number of elements associated with C<successorClause>.

=cut
sub count_successorClause ($)
{
  my ($self) = @_;

  my $x = $self->{'successorClause'};

    defined $x ? $x->size : 0;
  }




=for html <hr/>

=cut

#################################################################
# AssociationEnd  <---> test
# type = UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action
# multiplicity = 1
# ordering = 

=head2 C<test>

  my $val = $obj->test;

Returns the AssociationEnd C<test> value of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action>.

=cut
sub test ($)
{
  my ($self) = @_;
		  
  $self->{'test'};
}


=head2 C<set_test>

  $obj->set_test($val);

Sets the AssociationEnd C<test> value.
C<$val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action>.
Returns C<$obj>.

=cut
sub set_test ($$)
{
  my ($self, $val) = @_;
		  
  no warnings; # Use of uninitialized value in string ne at ...
		  
  my $old;
  if ( ($old = $self->{'test'}) ne $val ) { # Recursion lock

    if ( defined $val ) { $self->__use('UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action')->__typecheck($val, "UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause.test") }

    # Recursion lock
        $self->{'test'} = $val
    ;

    # Remove and add associations with other ends.
          }
		  
  $self;
}


=head2 C<add_test>

  $obj->add_test($val);

Adds the AssociationEnd C<test> value.
C<$val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action>.
Throws exception if a value already exists.
Returns C<$obj>.

=cut
sub add_test ($$)
{
  my ($self, $val) = @_;

  no warnings; # Use of uninitialized value in string ne at ...

  my $old;
  if ( ($old = $self->{'test'}) ne $val ) { # Recursion lock
    $self->__use('UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action')->__typecheck($val, "UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause.test");
      
    # confess("UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause::test: too many")
    # if defined $self->{'test'};

    # Recursion lock
        $self->{'test'} = $val
    ;

    # Remove and add associations with other ends.
        
  }

  $self;
}


=head2 C<remove_test>

  $obj->remove_test($val);

Removes the AssociationEnd C<test> value C<$val>.
Returns C<$obj>.

=cut
sub remove_test ($$)
{
  my ($self, $val) = @_;

  no warnings; # Use of uninitialized value in string ne at ...

  my $old;
  if ( ($old = $self->{'test'}) eq $val ) { # Recursion lock
    $val = $self->{'test'} = undef;         # Recursion lock

    # Remove and add associations with other ends.
        
  }
}


=head2 C<clear_test>

  $obj->clear_test;

Clears the AssociationEnd C<test> links to L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action>.
Returns C<$obj>.

=cut
sub clear_test ($@)
{
  my ($self) = @_;

  my $old;
  if ( defined ($old = $self->{'test'}) ) { # Recursion lock
    my $val = $self->{'test'} = undef;      # Recursion lock

    # Remove and add associations with other ends.
          }

  $self;
}


=head2 C<count_test>

  $obj->count_test;

Returns the number of elements of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::Action> associated with C<test>.

=cut
sub count_test ($)
{
  my ($self) = @_;

  my $x = $self->{'test'};

  defined $x ? 1 : 0;
}




=for html <hr/>

=cut

#################################################################
# AssociationEnd  <---> testOutput
# type = UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin
# multiplicity = 0..*
# ordering = 

=head2 C<testOutput>

  my @val = $obj->testOutput;
  my $ary_val = $obj->testOutput;

Returns the AssociationEnd C<testOutput> values of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin>.
In array context, returns all the objects in the Association.
In scalar context, returns an array ref of all the objects in the Association.

=cut
sub testOutput ($)
{
  my ($self) = @_;

    my $x = $self->{'testOutput'};

  # confess("Container for testOutput $x is not a blessed ref: " . Data::Dumper->new([ $self ], [qw($self)])->Maxdepth(2)->Dump()) if $x && ref($x) !~ /::/;
 
  wantarray ? ($x ? $x->members() : ()) : [ $x ? $x->members() : () ];
  
}


=head2 C<set_testOutput>

  $obj->set_testOutput(@val);

Sets the AssociationEnd C<testOutput> value.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin>.
Returns C<$obj>.

=cut
sub set_testOutput ($@)
{
  my ($self, @val) = @_;
  
  $self->clear_testOutput;
  $self->add_testOutput(@val);
}


=head2 C<add_testOutput>

  $obj->add_testOutput(@val);

Adds AssociationEnd C<testOutput> values.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin>.
Returns C<$obj>.

=cut
sub add_testOutput ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'testOutput'} ||= Set::Object->new();
    my $old; # Place holder for other MACRO.
  
  for my $val ( @val ) {
    # Recursion lock
        next if $x->includes($val);
        $self->__use('UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin')->__typecheck($val, "UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause.testOutput");

    # Recursion lock
        $x->insert($val);
    # weaken?
    
    # Remove and add associations with other ends.
          }
  
  $self;
}


=head2 C<remove_testOutput>

  $obj->remove_testOutput(@val);

Removes the AssociationEnd C<testOutput> values C<@val>.
Elements of C<@val> must of type L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin>.
Returns C<$obj>.

=cut
sub remove_testOutput ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'testOutput'} ||= Set::Object->new();
  
  for my $old ( @val ) {
    # Recursion lock
        next unless $x->includes($old);
    
    my $val = $old;
      
    $self->__use('UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin')->__typecheck($val, "UMMF::UML_1_5::Behavioral_Elements::Actions::Composite_Actions::Clause.testOutput");

    # Recursion lock
        $x->remove($old);
    
    $val = undef;

    # Remove associations with other ends.

        ;

  }
  
  $self;
}


=head2 C<clear_testOutput>

  $obj->clear_testOutput;

Clears the AssociationEnd C<testOutput> links to L<UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin|UMMF::UML_1_5::Behavioral_Elements::Actions::Action_Foundation::OutputPin>.
Returns C<$obj>.

=cut
sub clear_testOutput ($) 
{
  my ($self) = @_;
  
    my $x = $self->{'testOutput'} ||= Set::Object->new();
  
  my $val; # Place holder for other MACRO.
  
    $self->{'testOutput'} = Set::Object->new(); # Recursion lock
  for my $old ( $x->members() ) {     # Recursion lock
  
    # Remove associations with other ends.

        ;

  }
  
  $self;
}


=head2 C<count_testOutput>

  $obj->count_testOutput;

Returns the number of elements associated with C<testOutput>.

=cut
sub count_testOutput ($)
{
  my ($self) = @_;

  my $x = $self->{'testOutput'};

    defined $x ? $x->size : 0;
  }







# End of Class Clause


=pod

=for html <hr/>

I<END OF DOCUMENT>

=cut

############################################################################

1; # is true!

############################################################################

### Keep these comments at end of file: kstephens@users.sourceforge.net 2003/04/06 ###
### Local Variables: ###
### mode:perl ###
### perl-indent-level:2 ###
### perl-continued-statement-offset:0 ###
### perl-brace-offset:0 ###
### perl-label-offset:0 ###
### End: ###

