# -*- perl -*-
# DO NOT EDIT - This file is generated by UMMF; http://ummf.sourceforge.net 
# From template: $Id: Perl.txt,v 1.77 2006/05/14 01:40:03 kstephens Exp $

package UMMF::UML_1_5::OCL::Expressions::OperationCallExp;

#use 5.6.1;
use strict;
use warnings;

#################################################################
# Version
#

our $VERSION = do { my @r = (q{1.5} =~ /\d+/g); sprintf "%d." . "%03d" x $#r, @r };


#################################################################
# Documentation
#

=head1 NAME

UMMF::UML_1_5::OCL::Expressions::OperationCallExp -- 

=head1 VERSION

1.5

=head1 SYNOPSIS

=head1 DESCRIPTION 

=head1 USAGE

=head1 EXPORT

=head1 METATYPE

L<UMMF::UML_1_5::Foundation::Core::Class|UMMF::UML_1_5::Foundation::Core::Class>

=head1 SUPERCLASSES

L<UMMF::UML_1_5::OCL::Expressions::ModelPropertyCallExp|UMMF::UML_1_5::OCL::Expressions::ModelPropertyCallExp>




=head1 ATTRIBUTES

I<NO ATTRIBUTES>


=head1 ASSOCIATIONS


=head2 C<parentOperation> : I<THIS> C<0..1> E<lt>---E<gt>  C<arguments> : UMMF::UML_1_5::OCL::Expressions::OclExpression C<0..*>



=over 4

=item metatype = L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML_1_5::OCL::Expressions::OclExpression|UMMF::UML_1_5::OCL::Expressions::OclExpression>

=item multiplicity = C<0..*>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<ordered>

=item isNavigable = C<1>

=item aggregation = C<none>

=item visibility = C<public>

=item container_type = C<ARRAY>

=back


=head2 C<> : I<THIS> C<0..*> ----E<gt>  C<referredOperation> : UMMF::UML_1_5::Foundation::Core::Operation C<1>



=over 4

=item metatype = L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML_1_5::Foundation::Core::Operation|UMMF::UML_1_5::Foundation::Core::Operation>

=item multiplicity = C<1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<1>

=item aggregation = C<none>

=item visibility = C<public>

=item container_type = C<Set::Object>

=back



=head1 METHODS

=cut



#################################################################
# Dependencies
#





use Carp qw(croak confess);
use Set::Object 1.05;
use Class::Multimethods 1.70;
use Data::Dumper;
use Scalar::Util qw(weaken);
use UMMF::UML_1_5::__ObjectBase qw(:__ummf_array);


#################################################################
# Generalizations
#

use base qw(
  UMMF::UML_1_5::OCL::Expressions::ModelPropertyCallExp



);


#################################################################
# Exports
#

our @EXPORT_OK = qw(
);
our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );





#################################################################
# Validation
#


=head2 C<__validate_type>

  UMMF::UML_1_5::OCL::Expressions::OperationCallExp->__validate_type($value);

Returns true if C<$value> is a valid representation of L<UMMF::UML_1_5::OCL::Expressions::OperationCallExp|UMMF::UML_1_5::OCL::Expressions::OperationCallExp>.

=cut
sub __validate_type($$)
{
  my ($self, $x) = @_;

  no warnings;

  UNIVERSAL::isa($x, 'UMMF::UML_1_5::OCL::Expressions::OperationCallExp')  ;
}


=head2 C<__typecheck>

  UMMF::UML_1_5::OCL::Expressions::OperationCallExp->__typecheck($value, $msg);

Calls C<confess()> with C<$msg> if C<<UMMF::UML_1_5::OCL::Expressions::OperationCallExp->__validate_type($value)>> is false.

=cut
sub __typecheck
{
  my ($self, $x, $msg) = @_;

  confess("typecheck: $msg: type '" . 'UMMF::UML_1_5::OCL::Expressions::OperationCallExp' . ": value '$x'")
    unless __validate_type($self, $x);
}


=head2 C<isaOperationCallExp>


Returns true if receiver is a L<UMMF::UML_1_5::OCL::Expressions::OperationCallExp|UMMF::UML_1_5::OCL::Expressions::OperationCallExp>.
Other receivers will return false.

=cut
sub isaOperationCallExp { 1 }


=head2 C<isaOCL__Expressions__OperationCallExp>


Returns true if receiver is a L<UMMF::UML_1_5::OCL::Expressions::OperationCallExp|UMMF::UML_1_5::OCL::Expressions::OperationCallExp>.
Other receivers will return false.
This is the fully qualified version of the C<isaOperationCallExp> method.

=cut
sub isaOCL__Expressions__OperationCallExp { 1 }


#################################################################
# Introspection
#

=head2 C<__model_name> 

  my $name = $obj_or_package->__model_name;

Returns the UML Model name (C<'OCL::Expressions::OperationCallExp'>) for an object or package of
this Classifier.

=cut
sub __model_name { 'OCL::Expressions::OperationCallExp' }



=head2 C<__isAbstract>

  $package->__isAbstract;

Returns C<0>.

=cut
sub __isAbstract { 0; }


my $__tangram_schema;
=head2 C<__tangram_schema>

  my $tangram_schema $obj_or_package->__tangram_schema

Returns a HASH ref that describes this Classifier for Tangram.

See L<UMMF::Export::Perl::Tangram|UMMF::Export::Perl::Tangram>

=cut
sub __tangram_schema
{
  my ($self) = @_;

  $__tangram_schema ||=
  {
   'classes' =>
   [
     'UMMF::UML_1_5::OCL::Expressions::OperationCallExp' =>
     {
       'table' => 'OCL__Expressions__OperationCallExp',
       'abstract' => 0,
       'slots' => 
       { 
	 # Attributes
	 
	 # Associations
	 	 	       'arguments'
       => {
	 'type_impl' => 'iarray',
         'class' => 'UMMF::UML_1_5::OCL::Expressions::OclExpression',

                           'table' => 'OCL__Expressions__OperationCallExp__arguments', 

                                                               'coll' => 'parentOperation',

                  'slot' => 'parentOperation_i', 

                                    'aggreg' => '1', 

                                           }
      ,
                  	 	       'referredOperation'
       => {
	 'type_impl' => 'ref',
         'class' => 'UMMF::UML_1_5::Foundation::Core::Operation',

                                             'col' => 'referredOperation', 

                                                                                                                   }
      ,
                         },
       'bases' => [  'UMMF::UML_1_5::OCL::Expressions::ModelPropertyCallExp',  ],
       'sql' => {

       },
     },
   ],

   'sql' =>
   {
    # Note Tangram::Ref::get_exporter() has
    # "UPDATE $table SET $self->{col} = $refid WHERE id = $id",
    # The id_col is hard-coded, 
    # Thus id_col will not work.
    #'id_col' => '__sid',
    #'class_col' => '__stype',
   },
     # 'set_id' => sub { }
     # 'get_id' => sub { }

      
  };
}


#################################################################
# Class Attributes
#


    

#################################################################
# Class Associations
#


    

#################################################################
# Initialization
#


=head2 C<___initialize>

Initialize all Attributes and AssociationEnds in a instance of this Classifier.
Does B<not> initalize slots in its Generalizations.

See also: C<__initialize>.

=cut
sub ___initialize
{
  my ($self) = @_;

  # Attributes



  # Associations

  # AssociationEnd 
  #  parentOperation 0..1
  #  <--> 
  #  arguments 0..* UMMF::UML_1_5::OCL::Expressions::OclExpression.
    if ( defined $self->{'arguments'} ) {
    my $x = $self->{'arguments'};
        $self->{'arguments'} = [ ];
        $self->set_arguments(@$x);
  }
  
  # AssociationEnd 
  #   0..*
  #  <--> 
  #  referredOperation 1 UMMF::UML_1_5::Foundation::Core::Operation.
    if ( defined $self->{'referredOperation'} ) {
    my $x = $self->{'referredOperation'};
    $self->{'referredOperation'} = undef;
    $self->set_referredOperation($x);
  }
  

  $self;
}


my $__initialize_use;

=head2 C<__initialize>

Initialize all slots in this Classifier and all its Generalizations.

See also: C<___initialize>.

=cut
sub __initialize
{
  my ($self) = @_;

  # $DB::single = 1;

  unless ( ! $__initialize_use ) {
    $__initialize_use = 1;
    $self->__use('UMMF::UML_1_5::Foundation::Core::Element');
    $self->__use('UMMF::UML_1_5::Foundation::Core::ModelElement');
    $self->__use('UMMF::UML_1_5::OCL::Expressions::OclExpression');
    $self->__use('UMMF::UML_1_5::OCL::Expressions::PropertyCallExp');
    $self->__use('UMMF::UML_1_5::OCL::Expressions::ModelPropertyCallExp');
  }

  $self->UMMF::UML_1_5::OCL::Expressions::OperationCallExp::___initialize;
  $self->UMMF::UML_1_5::Foundation::Core::Element::___initialize;
  $self->UMMF::UML_1_5::Foundation::Core::ModelElement::___initialize;
  $self->UMMF::UML_1_5::OCL::Expressions::OclExpression::___initialize;
  $self->UMMF::UML_1_5::OCL::Expressions::PropertyCallExp::___initialize;
  $self->UMMF::UML_1_5::OCL::Expressions::ModelPropertyCallExp::___initialize;

  $self;
}
      

=head2 C<__create>

Calls all <<create>> Methods for this Classifier and all Generalizations.

See also: C<___create>.

=cut
sub __create
{
  my ($self, @args) = @_;

  # $DB::single = 1;
  $self->UMMF::UML_1_5::OCL::Expressions::OperationCallExp::___create(@args);
  $self->UMMF::UML_1_5::Foundation::Core::Element::___create();
  $self->UMMF::UML_1_5::Foundation::Core::ModelElement::___create();
  $self->UMMF::UML_1_5::OCL::Expressions::OclExpression::___create();
  $self->UMMF::UML_1_5::OCL::Expressions::PropertyCallExp::___create();
  $self->UMMF::UML_1_5::OCL::Expressions::ModelPropertyCallExp::___create();

  $self;
}




#################################################################
# Attributes
#




#################################################################
# Association
#


=for html <hr/>

=cut

#################################################################
# AssociationEnd parentOperation <---> arguments
# type = UMMF::UML_1_5::OCL::Expressions::OclExpression
# multiplicity = 0..*
# ordering = ordered

=head2 C<arguments>

  my @val = $obj->arguments;
  my $ary_val = $obj->arguments;

Returns the AssociationEnd C<arguments> values of type L<UMMF::UML_1_5::OCL::Expressions::OclExpression|UMMF::UML_1_5::OCL::Expressions::OclExpression>.
In array context, returns all the objects in the Association.
In scalar context, returns an array ref of all the objects in the Association.

=cut
sub arguments ($)
{
  my ($self) = @_;

    my $x = $self->{'arguments'} ||= [ ];

  wantarray ? @{$x} : $x;
  
}


=head2 C<index_arguments>

  my $x = $obj->index_arguments($i);
  my @x = $obj->index_arguments($i, $count);

In scalar context, returns the value of AssociationEnd C<arguments> at index C<$i>.
In array context, returns the values between index C<$i> and C<$i + $count - 1>, inclusive.

=cut
sub index_arguments ($$@)
{
  my ($self, $i, $count) = @_;

  ;

  my $val = $self->{'arguments'} ||= [ ];

  ;

  wantarray ? $val->[$i .. (defined $count ? $i + $count - 1 : $i)]
            : $val->[$i];
}


=head2 C<index_of_arguments>

  my $index = $obj->index_of_arguments($val);

Returns the index of C<$val> in AssociationEnd C<arguments>.
Return C<undef> if C<$val> is not in C<arguments>.

=cut
sub index_of_arguments ($$)
{
  my ($self, $x) = @_;

  ;

  my $val = $self->{'arguments'} ||= [ ];

  ;

  __ummf_array_index($val, $x);
}


=head2 C<set_arguments>

  $obj->set_arguments(@val);

Sets the AssociationEnd C<arguments> value.
Elements of C<@val> must of type L<UMMF::UML_1_5::OCL::Expressions::OclExpression|UMMF::UML_1_5::OCL::Expressions::OclExpression>.
Returns C<$obj>.

=cut
sub set_arguments ($@)
{
  my ($self, @val) = @_;
  
  $self->clear_arguments;
  $self->add_arguments(@val);
}


=head2 C<set_index_arguments>

  $obj->set_index_arguments($i, $val);

Sets the value of AssociationEnd C<arguments> at index C<$i>.
Returns self.

=cut
sub set_index_arguments ($$$)
{
  my ($self, $i, $val) = @_;

  ;

  my $x = $self->{'arguments'} ||= [ ];

  no warnings;
  my $old;
  if ( ($old = $x->[$i]) ne $val) {
    # Recursion lock
        $x->[$i] = $val
    ;

    # Remove and add associations with other ends.
        
    $old->remove_parentOperation($self) if $old;
    $val->add_parentOperation($self)    if $val;

  
    ;
  }

  $self;
}


=head2 C<add_arguments>

  $obj->add_arguments(@val);

Adds AssociationEnd C<arguments> values.
Elements of C<@val> must of type L<UMMF::UML_1_5::OCL::Expressions::OclExpression|UMMF::UML_1_5::OCL::Expressions::OclExpression>.
Returns C<$obj>.

=cut
sub add_arguments ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'arguments'} ||= [ ];
    my $old; # Place holder for other MACRO.
  
  for my $val ( @val ) {
    # Recursion lock
        next if grep($_ eq $val, @$x);
        $self->__use('UMMF::UML_1_5::OCL::Expressions::OclExpression')->__typecheck($val, "UMMF::UML_1_5::OCL::Expressions::OperationCallExp.arguments");

    # Recursion lock
        push(@{$x}, $val);
        
    # Remove and add associations with other ends.
        
    $old->remove_parentOperation($self) if $old;
    $val->add_parentOperation($self)    if $val;

    }
  
  $self;
}


=head2 C<add_index_arguments>

  $obj->add_index_arguments($i, @val);

Adds AssociationEnd C<arguments> values at index C<$i>.
Elements of C<@val> must of type L<UMMF::UML_1_5::OCL::Expressions::OclExpression|UMMF::UML_1_5::OCL::Expressions::OclExpression>.
Returns C<$obj>.

=cut
sub add_index_arguments ($$@)
{
  my ($self, $i, @val) = @_;

  
    my $x = $self->{'arguments'} ||= [ ];
    my $old; # Place holder for other MACRO.
  
  for my $val ( @val ) {
    # Recursion lock
        next if grep($_ eq $val, @$x);
        $self->__use('UMMF::UML_1_5::OCL::Expressions::OclExpression')->__typecheck($val, "UMMF::UML_1_5::OCL::Expressions::OperationCallExp.arguments");

    # Recursion lock
        splice(@{$x}, $i, 0, $val); # Recursion lock
        ++ $i;
    
    # Remove and add associations with other ends.
        
    $old->remove_parentOperation($self) if $old;
    $val->add_parentOperation($self)    if $val;

    }
  
  
  $self;
}


=head2 C<remove_arguments>

  $obj->remove_arguments(@val);

Removes the AssociationEnd C<arguments> values C<@val>.
Elements of C<@val> must of type L<UMMF::UML_1_5::OCL::Expressions::OclExpression|UMMF::UML_1_5::OCL::Expressions::OclExpression>.
Returns C<$obj>.

=cut
sub remove_arguments ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'arguments'} ||= [ ];
  
  for my $old ( @val ) {
    # Recursion lock
        my $i; # index of $old in @$x.
    next unless defined($i = __ummf_array_index($x, $old));
    
    my $val = $old;
      
    $self->__use('UMMF::UML_1_5::OCL::Expressions::OclExpression')->__typecheck($val, "UMMF::UML_1_5::OCL::Expressions::OperationCallExp.arguments");

    # Recursion lock
        splice(@$x, $i, 1);     
    $val = undef;

    # Remove associations with other ends.

        
    $old->remove_parentOperation($self) if $old;
    $val->add_parentOperation($self)    if $val;

  ;

  }
  
  $self;
}


=head2 C<clear_arguments>

  $obj->clear_arguments;

Clears the AssociationEnd C<arguments> links to L<UMMF::UML_1_5::OCL::Expressions::OclExpression|UMMF::UML_1_5::OCL::Expressions::OclExpression>.
Returns C<$obj>.

=cut
sub clear_arguments ($) 
{
  my ($self) = @_;
  
    my $x = $self->{'arguments'} ||= [ ];
  
  my $val; # Place holder for other MACRO.
  
    $self->{'arguments'} = [ ];  # Recursion lock
  for my $old ( @$x ) { # Recursion lock
  
    # Remove associations with other ends.

        
    $old->remove_parentOperation($self) if $old;
    $val->add_parentOperation($self)    if $val;

  ;

  }
  
  $self;
}


=head2 C<count_arguments>

  $obj->count_arguments;

Returns the number of elements associated with C<arguments>.

=cut
sub count_arguments ($)
{
  my ($self) = @_;

  my $x = $self->{'arguments'};

    defined $x ? scalar @$x : 0;
  }




=for html <hr/>

=cut

#################################################################
# AssociationEnd  <---> referredOperation
# type = UMMF::UML_1_5::Foundation::Core::Operation
# multiplicity = 1
# ordering = 

=head2 C<referredOperation>

  my $val = $obj->referredOperation;

Returns the AssociationEnd C<referredOperation> value of type L<UMMF::UML_1_5::Foundation::Core::Operation|UMMF::UML_1_5::Foundation::Core::Operation>.

=cut
sub referredOperation ($)
{
  my ($self) = @_;
		  
  $self->{'referredOperation'};
}


=head2 C<set_referredOperation>

  $obj->set_referredOperation($val);

Sets the AssociationEnd C<referredOperation> value.
C<$val> must of type L<UMMF::UML_1_5::Foundation::Core::Operation|UMMF::UML_1_5::Foundation::Core::Operation>.
Returns C<$obj>.

=cut
sub set_referredOperation ($$)
{
  my ($self, $val) = @_;
		  
  no warnings; # Use of uninitialized value in string ne at ...
		  
  my $old;
  if ( ($old = $self->{'referredOperation'}) ne $val ) { # Recursion lock

    if ( defined $val ) { $self->__use('UMMF::UML_1_5::Foundation::Core::Operation')->__typecheck($val, "UMMF::UML_1_5::OCL::Expressions::OperationCallExp.referredOperation") }

    # Recursion lock
        $self->{'referredOperation'} = $val
    ;

    # Remove and add associations with other ends.
          }
		  
  $self;
}


=head2 C<add_referredOperation>

  $obj->add_referredOperation($val);

Adds the AssociationEnd C<referredOperation> value.
C<$val> must of type L<UMMF::UML_1_5::Foundation::Core::Operation|UMMF::UML_1_5::Foundation::Core::Operation>.
Throws exception if a value already exists.
Returns C<$obj>.

=cut
sub add_referredOperation ($$)
{
  my ($self, $val) = @_;

  no warnings; # Use of uninitialized value in string ne at ...

  my $old;
  if ( ($old = $self->{'referredOperation'}) ne $val ) { # Recursion lock
    $self->__use('UMMF::UML_1_5::Foundation::Core::Operation')->__typecheck($val, "UMMF::UML_1_5::OCL::Expressions::OperationCallExp.referredOperation");
      
    # confess("UMMF::UML_1_5::OCL::Expressions::OperationCallExp::referredOperation: too many")
    # if defined $self->{'referredOperation'};

    # Recursion lock
        $self->{'referredOperation'} = $val
    ;

    # Remove and add associations with other ends.
        
  }

  $self;
}


=head2 C<remove_referredOperation>

  $obj->remove_referredOperation($val);

Removes the AssociationEnd C<referredOperation> value C<$val>.
Returns C<$obj>.

=cut
sub remove_referredOperation ($$)
{
  my ($self, $val) = @_;

  no warnings; # Use of uninitialized value in string ne at ...

  my $old;
  if ( ($old = $self->{'referredOperation'}) eq $val ) { # Recursion lock
    $val = $self->{'referredOperation'} = undef;         # Recursion lock

    # Remove and add associations with other ends.
        
  }
}


=head2 C<clear_referredOperation>

  $obj->clear_referredOperation;

Clears the AssociationEnd C<referredOperation> links to L<UMMF::UML_1_5::Foundation::Core::Operation|UMMF::UML_1_5::Foundation::Core::Operation>.
Returns C<$obj>.

=cut
sub clear_referredOperation ($@)
{
  my ($self) = @_;

  my $old;
  if ( defined ($old = $self->{'referredOperation'}) ) { # Recursion lock
    my $val = $self->{'referredOperation'} = undef;      # Recursion lock

    # Remove and add associations with other ends.
          }

  $self;
}


=head2 C<count_referredOperation>

  $obj->count_referredOperation;

Returns the number of elements of type L<UMMF::UML_1_5::Foundation::Core::Operation|UMMF::UML_1_5::Foundation::Core::Operation> associated with C<referredOperation>.

=cut
sub count_referredOperation ($)
{
  my ($self) = @_;

  my $x = $self->{'referredOperation'};

  defined $x ? 1 : 0;
}







# End of Class OperationCallExp


=pod

=for html <hr/>

I<END OF DOCUMENT>

=cut

############################################################################

1; # is true!

############################################################################

### Keep these comments at end of file: kstephens@users.sourceforge.net 2003/04/06 ###
### Local Variables: ###
### mode:perl ###
### perl-indent-level:2 ###
### perl-continued-statement-offset:0 ###
### perl-brace-offset:0 ###
### perl-label-offset:0 ###
### End: ###

