# -*- perl -*-
# DO NOT EDIT - This file is generated by UMMF; http://ummf.sourceforge.net 
# From template: $Id: Perl.txt,v 1.77 2006/05/14 01:40:03 kstephens Exp $

package UMMF::MOF_1_3::Model::Exception;

#use 5.6.1;
use strict;
use warnings;

#################################################################
# Version
#

our $VERSION = do { my @r = (q{1.3} =~ /\d+/g); sprintf "%d." . "%03d" x $#r, @r };


#################################################################
# Documentation
#

=head1 NAME

UMMF::MOF_1_3::Model::Exception -- 

=head1 VERSION

1.3

=head1 SYNOPSIS

=head1 DESCRIPTION 

=head1 USAGE

=head1 EXPORT

=head1 METATYPE

L<UMMF::UML_1_5::Foundation::Core::Class|UMMF::UML_1_5::Foundation::Core::Class>

=head1 SUPERCLASSES

L<UMMF::MOF_1_3::Model::BehavioralFeature|UMMF::MOF_1_3::Model::BehavioralFeature>




=head1 ATTRIBUTES

I<NO ATTRIBUTES>


=head1 ASSOCIATIONS


=head2 C<except> : I<THIS> C<0..*> E<lt>---E<gt>  C<operation> : UMMF::MOF_1_3::Model::Operation C<0..*>



=over 4

=item metatype = L<UMMF::UML_1_5::Foundation::Core::AssociationEnd|UMMF::UML_1_5::Foundation::Core::AssociationEnd>

=item type = L<UMMF::MOF_1_3::Model::Operation|UMMF::MOF_1_3::Model::Operation>

=item multiplicity = C<0..*>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<1>

=item aggregation = C<none>

=item visibility = C<public>

=item container_type = C<Set::Object>

=back



=head1 METHODS

=cut



#################################################################
# Dependencies
#





use Carp qw(croak confess);
use Set::Object 1.05;
use Class::Multimethods 1.70;
use Data::Dumper;
use Scalar::Util qw(weaken);
use UMMF::MOF_1_3::__ObjectBase qw(:__ummf_array);


#################################################################
# Generalizations
#

use base qw(
  UMMF::MOF_1_3::Model::BehavioralFeature



);


#################################################################
# Exports
#

our @EXPORT_OK = qw(
);
our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );





#################################################################
# Validation
#


=head2 C<__validate_type>

  UMMF::MOF_1_3::Model::Exception->__validate_type($value);

Returns true if C<$value> is a valid representation of L<UMMF::MOF_1_3::Model::Exception|UMMF::MOF_1_3::Model::Exception>.

=cut
sub __validate_type($$)
{
  my ($self, $x) = @_;

  no warnings;

  UNIVERSAL::isa($x, 'UMMF::MOF_1_3::Model::Exception')  ;
}


=head2 C<__typecheck>

  UMMF::MOF_1_3::Model::Exception->__typecheck($value, $msg);

Calls C<confess()> with C<$msg> if C<<UMMF::MOF_1_3::Model::Exception->__validate_type($value)>> is false.

=cut
sub __typecheck
{
  my ($self, $x, $msg) = @_;

  confess("typecheck: $msg: type '" . 'UMMF::MOF_1_3::Model::Exception' . ": value '$x'")
    unless __validate_type($self, $x);
}


=head2 C<isaException>


Returns true if receiver is a L<UMMF::MOF_1_3::Model::Exception|UMMF::MOF_1_3::Model::Exception>.
Other receivers will return false.

=cut
sub isaException { 1 }


=head2 C<isaModel__Exception>


Returns true if receiver is a L<UMMF::MOF_1_3::Model::Exception|UMMF::MOF_1_3::Model::Exception>.
Other receivers will return false.
This is the fully qualified version of the C<isaException> method.

=cut
sub isaModel__Exception { 1 }


#################################################################
# Introspection
#

=head2 C<__model_name> 

  my $name = $obj_or_package->__model_name;

Returns the UML Model name (C<'Model::Exception'>) for an object or package of
this Classifier.

=cut
sub __model_name { 'Model::Exception' }



=head2 C<__isAbstract>

  $package->__isAbstract;

Returns C<0>.

=cut
sub __isAbstract { 0; }


my $__tangram_schema;
=head2 C<__tangram_schema>

  my $tangram_schema $obj_or_package->__tangram_schema

Returns a HASH ref that describes this Classifier for Tangram.

See L<UMMF::Export::Perl::Tangram|UMMF::Export::Perl::Tangram>

=cut
sub __tangram_schema
{
  my ($self) = @_;

  $__tangram_schema ||=
  {
   'classes' =>
   [
     'UMMF::MOF_1_3::Model::Exception' =>
     {
       'table' => 'Model__Exception',
       'abstract' => 0,
       'slots' => 
       { 
	 # Attributes
	 
	 # Associations
	 	 	       'operation'
       => {
	 'type_impl' => 'set',
         'class' => 'UMMF::MOF_1_3::Model::Operation',

                           'table' => 'Model__CanRaise', 

                                                      'item' => 'operation', 

                  'coll' => 'except',

                                                                               }
      ,
                         },
       'bases' => [  'UMMF::MOF_1_3::Model::BehavioralFeature',  ],
       'sql' => {

       },
     },
   ],

   'sql' =>
   {
    # Note Tangram::Ref::get_exporter() has
    # "UPDATE $table SET $self->{col} = $refid WHERE id = $id",
    # The id_col is hard-coded, 
    # Thus id_col will not work.
    #'id_col' => '__sid',
    #'class_col' => '__stype',
   },
     # 'set_id' => sub { }
     # 'get_id' => sub { }

      
  };
}


#################################################################
# Class Attributes
#


    

#################################################################
# Class Associations
#


    

#################################################################
# Initialization
#


=head2 C<___initialize>

Initialize all Attributes and AssociationEnds in a instance of this Classifier.
Does B<not> initalize slots in its Generalizations.

See also: C<__initialize>.

=cut
sub ___initialize
{
  my ($self) = @_;

  # Attributes



  # Associations

  # AssociationEnd 
  #  except 0..*
  #  <--> 
  #  operation 0..* UMMF::MOF_1_3::Model::Operation.
    if ( defined $self->{'operation'} ) {
    my $x = $self->{'operation'};
        $self->{'operation'} = Set::Object->new();
        $self->set_operation(@$x);
  }
  

  $self;
}


my $__initialize_use;

=head2 C<__initialize>

Initialize all slots in this Classifier and all its Generalizations.

See also: C<___initialize>.

=cut
sub __initialize
{
  my ($self) = @_;

  # $DB::single = 1;

  unless ( ! $__initialize_use ) {
    $__initialize_use = 1;
    $self->__use('UMMF::MOF_1_3::Model::ModelElement');
    $self->__use('UMMF::MOF_1_3::Model::Feature');
    $self->__use('UMMF::MOF_1_3::Model::BehavioralFeature');
  }

  $self->UMMF::MOF_1_3::Model::Exception::___initialize;
  $self->UMMF::MOF_1_3::Model::ModelElement::___initialize;
  $self->UMMF::MOF_1_3::Model::Feature::___initialize;
  $self->UMMF::MOF_1_3::Model::BehavioralFeature::___initialize;

  $self;
}
      

=head2 C<__create>

Calls all <<create>> Methods for this Classifier and all Generalizations.

See also: C<___create>.

=cut
sub __create
{
  my ($self, @args) = @_;

  # $DB::single = 1;
  $self->UMMF::MOF_1_3::Model::Exception::___create(@args);
  $self->UMMF::MOF_1_3::Model::ModelElement::___create();
  $self->UMMF::MOF_1_3::Model::Feature::___create();
  $self->UMMF::MOF_1_3::Model::BehavioralFeature::___create();

  $self;
}




#################################################################
# Attributes
#




#################################################################
# Association
#


=for html <hr/>

=cut

#################################################################
# AssociationEnd except <---> operation
# type = UMMF::MOF_1_3::Model::Operation
# multiplicity = 0..*
# ordering = 

=head2 C<operation>

  my @val = $obj->operation;
  my $ary_val = $obj->operation;

Returns the AssociationEnd C<operation> values of type L<UMMF::MOF_1_3::Model::Operation|UMMF::MOF_1_3::Model::Operation>.
In array context, returns all the objects in the Association.
In scalar context, returns an array ref of all the objects in the Association.

=cut
sub operation ($)
{
  my ($self) = @_;

    my $x = $self->{'operation'};

  # confess("Container for operation $x is not a blessed ref: " . Data::Dumper->new([ $self ], [qw($self)])->Maxdepth(2)->Dump()) if $x && ref($x) !~ /::/;
 
  wantarray ? ($x ? $x->members() : ()) : [ $x ? $x->members() : () ];
  
}


=head2 C<set_operation>

  $obj->set_operation(@val);

Sets the AssociationEnd C<operation> value.
Elements of C<@val> must of type L<UMMF::MOF_1_3::Model::Operation|UMMF::MOF_1_3::Model::Operation>.
Returns C<$obj>.

=cut
sub set_operation ($@)
{
  my ($self, @val) = @_;
  
  $self->clear_operation;
  $self->add_operation(@val);
}


=head2 C<add_operation>

  $obj->add_operation(@val);

Adds AssociationEnd C<operation> values.
Elements of C<@val> must of type L<UMMF::MOF_1_3::Model::Operation|UMMF::MOF_1_3::Model::Operation>.
Returns C<$obj>.

=cut
sub add_operation ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'operation'} ||= Set::Object->new();
    my $old; # Place holder for other MACRO.
  
  for my $val ( @val ) {
    # Recursion lock
        next if $x->includes($val);
        $self->__use('UMMF::MOF_1_3::Model::Operation')->__typecheck($val, "UMMF::MOF_1_3::Model::Exception.operation");

    # Recursion lock
        $x->insert($val);
    # weaken?
    
    # Remove and add associations with other ends.
        
    $old->remove_except($self) if $old;
    $val->add_except($self)    if $val;

    }
  
  $self;
}


=head2 C<remove_operation>

  $obj->remove_operation(@val);

Removes the AssociationEnd C<operation> values C<@val>.
Elements of C<@val> must of type L<UMMF::MOF_1_3::Model::Operation|UMMF::MOF_1_3::Model::Operation>.
Returns C<$obj>.

=cut
sub remove_operation ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'operation'} ||= Set::Object->new();
  
  for my $old ( @val ) {
    # Recursion lock
        next unless $x->includes($old);
    
    my $val = $old;
      
    $self->__use('UMMF::MOF_1_3::Model::Operation')->__typecheck($val, "UMMF::MOF_1_3::Model::Exception.operation");

    # Recursion lock
        $x->remove($old);
    
    $val = undef;

    # Remove associations with other ends.

        
    $old->remove_except($self) if $old;
    $val->add_except($self)    if $val;

  ;

  }
  
  $self;
}


=head2 C<clear_operation>

  $obj->clear_operation;

Clears the AssociationEnd C<operation> links to L<UMMF::MOF_1_3::Model::Operation|UMMF::MOF_1_3::Model::Operation>.
Returns C<$obj>.

=cut
sub clear_operation ($) 
{
  my ($self) = @_;
  
    my $x = $self->{'operation'} ||= Set::Object->new();
  
  my $val; # Place holder for other MACRO.
  
    $self->{'operation'} = Set::Object->new(); # Recursion lock
  for my $old ( $x->members() ) {     # Recursion lock
  
    # Remove associations with other ends.

        
    $old->remove_except($self) if $old;
    $val->add_except($self)    if $val;

  ;

  }
  
  $self;
}


=head2 C<count_operation>

  $obj->count_operation;

Returns the number of elements associated with C<operation>.

=cut
sub count_operation ($)
{
  my ($self) = @_;

  my $x = $self->{'operation'};

    defined $x ? $x->size : 0;
  }







# End of Class Exception


=pod

=for html <hr/>

I<END OF DOCUMENT>

=cut

############################################################################

1; # is true!

############################################################################

### Keep these comments at end of file: kstephens@users.sourceforge.net 2003/04/06 ###
### Local Variables: ###
### mode:perl ###
### perl-indent-level:2 ###
### perl-continued-statement-offset:0 ###
### perl-brace-offset:0 ###
### perl-label-offset:0 ###
### End: ###

